@@ -1,6 +1,5 @@
-"""
-Exporter that will export your ipynb to Markdown.
-"""
+"""Markdown Exporter class"""
+
 #-----------------------------------------------------------------------------
 # Copyright (c) 2013, the IPython Development Team.
 #
@@ -14,25 +13,31 @@ Exporter that will export your ipynb to Markdown.
 #-----------------------------------------------------------------------------
 
 from IPython.config import Config
-from IPython.utils.traitlets import Unicode
 
-from .exporter import Exporter
+from .templateexporter import TemplateExporter
 
 #-----------------------------------------------------------------------------
 # Classes
 #-----------------------------------------------------------------------------
 
-class MarkdownExporter(Exporter):
+class MarkdownExporter(TemplateExporter):
     """
     Exports to a markdown document (.md)
     """
     
-    file_extension = Unicode(
-        'md', config=True, 
-        help="Extension of the file that should be written to disk")
+    def _file_extension_default(self):
+        return 'md'
+
+    def _template_file_default(self):
+        return 'markdown'
+
+    output_mimetype = 'text/markdown'
+    
+    def _raw_mimetypes_default(self):
+        return ['text/markdown', 'text/html', '']
 
     @property
     def default_config(self):
-        c = Config({'ExtractOutputTransformer':{'enabled':True}})
+        c = Config({'ExtractOutputPreprocessor':{'enabled':True}})
         c.merge(super(MarkdownExporter,self).default_config)
         return c
